﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Drawing.Imaging;

namespace eBoard
{
    public enum Action { Wait, Catch, Clear, New };
    public enum BoardType { Screen, White, Clip,  Desktop };
    public enum DrawMode { Path, Rect, Clip, Cursor };

    public class eBoard
    {
        public bool isMouseDown = false;       // 紀錄滑鼠是否被按下
        public List<Object> records = new List<Object>(); // 紀錄繪製動作的陣列。

        public ScreenCapture screenCapture = new ScreenCapture(); // 用來抓螢幕的物件。

        public Image screenImage = null; // 抓下來的全螢幕影像。
        public Path path = null;
        public Rect rect = null;
        public Clip clip = null;// 剪貼區域，包含影像

        public ArrayList colors = new ArrayList(new Color[] { Color.Red, Color.Green, Color.Blue, Color.Yellow, Color.White, Color.Black });
        public ArrayList actions = new ArrayList(new Action[] { Action.Wait, Action.Catch, Action.Clear, Action.New });
        public ArrayList boardTypes = new ArrayList(new BoardType[] { BoardType.Screen, BoardType.White, BoardType.Clip, BoardType.Desktop });
        public ArrayList drawModes = new ArrayList(new DrawMode[] { DrawMode.Path, DrawMode.Rect, DrawMode.Clip, DrawMode.Cursor });
        public ArrayList widths = new ArrayList(new int[] { 1, 2, 3, 4, 5, 6, 7, 8, 9 });

        public BoardType boardType = BoardType.White;
        public Action action = Action.Wait;
        public DrawMode drawMode = DrawMode.Path;
        public Pen pen = new Pen(Color.Black, 3); // 畫圖時使用的畫筆

        public Pen dashPen = new Pen(Color.Gray, 1); // Clip 時使用的畫筆。
        public Color bgColor = Color.White;    // 白色背景。


        public FormTool tool;
        public FormBoard board;

        static eBoard eboard = null;

        /// <summary>
        /// 應用程式的主要進入點。
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            eboard = new eBoard();
            Application.Run(eboard.tool);
        }

        public eBoard()
        {
            board = new FormBoard(this);
            tool = new FormTool(this);

            pen.LineJoin = System.Drawing.Drawing2D.LineJoin.Round; // 接點要圓弧狀。
            pen.StartCap = System.Drawing.Drawing2D.LineCap.Round; // 起始點要圓弧狀。
            dashPen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot; // 點狀畫筆。

            setPenColor(Color.Black);
            setPenWidth(3);
            setAction(Action.Wait);
            setBoardType(BoardType.Desktop);
            setDrawMode(DrawMode.Path);
        }

        public DrawMode lastDrawMode;

        public void setPenColor(Color pColor)
        {
            pen.Color = pColor;
            records.Add(pColor);
            tool.setPenColor(pColor);
        }

        public void setPenWidth(int pWidth)
        {
            pen.Width = pWidth;
            records.Add(new Width(pWidth));
            tool.setPenWidth(pWidth);
        }

        public void setDrawMode(DrawMode pMode)
        {
            lastDrawMode = drawMode;
            drawMode = pMode;
            records.Add(drawMode);
            tool.setDrawMode(pMode);
        }

        public void setBoardType(BoardType pType)
        {
            boardType = pType;
            tool.setBoardType(pType);
            board.redraw();
        }

        public void setAction(Action pAction)
        {
            action = pAction;
            tool.setAction(pAction);
        }

        public void keyHit(char pKey)
        {
            String keyStr = pKey.ToString().ToUpper();
            char key = keyStr[0];
            switch (key)
            {
                case 'P': setDrawMode(DrawMode.Path); break;
                case 'F': setDrawMode(DrawMode.Rect); break;
                case 'C': setDrawMode(DrawMode.Clip); break;
                case 'T': setAction(Action.Catch); break;
                case 'X': setAction(Action.Clear); break;
                case 'N': setAction(Action.New); break;
                case 'I': setAction(Action.Wait); break;
                case ' ': switchBoard(BoardType.White); break;
                case 'S': switchBoard(BoardType.Screen); break;
                case 'L': switchBoard(BoardType.Clip); break;
                case 'D': switchBoard(BoardType.Desktop); break;
                case 'R': setPenColor(Color.Red); break;
                case 'G': setPenColor(Color.Green); break;
                case 'B': setPenColor(Color.Blue); break;
                case 'K': setPenColor(Color.Black); break;
                case 'W': setPenColor(Color.White); break;
                case 'Y': setPenColor(Color.Yellow); break;
                case '1': setPenWidth(1); break;
                case '2': setPenWidth(2); break;
                case '3': setPenWidth(3); break;
                case '4': setPenWidth(4); break;
                case '5': setPenWidth(5); break;
                case '6': setPenWidth(6); break;
                case '7': setPenWidth(7); break;
                case '8': setPenWidth(8); break;
                case '9': setPenWidth(9); break;
            }
        }

        public Bitmap clipScreen(Rectangle r)
        {
            Bitmap screenBitmap = new Bitmap(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height);
            Graphics gScreen = Graphics.FromImage(screenBitmap);
            gScreen.Clear(bgColor);
            draw(gScreen);

            Bitmap toBitmap = new Bitmap(r.Width, r.Height);
            Graphics gTo = Graphics.FromImage(toBitmap);
            gTo.DrawImage(screenBitmap, new Rectangle(0, 0, r.Width, r.Height), new Rectangle(r.X, r.Y, r.Width, r.Height), GraphicsUnit.Pixel);
            return toBitmap;
        }

        public void whiteScreen()
        {
            screenImage = null;
            board.redraw();
            board.Show();
        }

        public void saveScreen(String fileName)
        {
            Bitmap bitmap = clipScreen(new Rectangle(0, 0, Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height));
            bitmap.Save(fileName, ImageFormat.Jpeg);
        }

        public void draw(Graphics g)
        {
            switch (boardType)
            {
                case BoardType.White: 
                    g.Clear(bgColor); 
                    break;
                case BoardType.Screen: 
                    if (screenImage != null)
                        g.DrawImage(screenImage, new Point(0, 0)); 
                    break;
                case BoardType.Clip:
                    if (clip != null && clip.image != null)
                        g.DrawImage(clip.image, new Point(clip.rect.r.X, clip.rect.r.Y));
                    break;
                case BoardType.Desktop:
                    break;
            }

            Pen tpen = (Pen)pen.Clone();

            for (int i = 0; i < records.Count; i++)
            {
                Object o = records[i];
                if (o is Path)
                {
                    ((Path)o).draw(tpen, g);
                }
                else if (o is DrawMode)
                {
                }
                else if (o is Width)
                {
                    tpen.Width = ((Width)o).width;
                }
                else if (o is Color)
                {
                    tpen.Color = (Color)o;
                }
                else if (o is Rect)
                {
                    ((Rect)o).draw(tpen, g);
                }
                else if (o is Clip)
                {
                    ((Clip)o).rect.draw(dashPen, g);
                }
            }
        }

        public void boardMouseDown(int x, int y)
        {

            isMouseDown = true; // 滑鼠被按下後設定旗標值。
            if (drawMode == DrawMode.Path)
            {
                path = new Path();
                path.add(new Point(x, y)); // 將點加入到 path 中
                records.Add(path);
            }
            else if (drawMode == DrawMode.Rect)
            {
                rect = new Rect(x, y, 1, 1);
                records.Add(rect);
            }
            else if (drawMode == DrawMode.Clip)
            {
                clip = new Clip();
                clip.rect = new Rect(x, y, 1, 1);
                records.Add(clip);
            }
            board.redraw();
        }

        public void boardMouseMove(int x, int y)
        {
            if (isMouseDown) // 如果滑鼠被按下
            {
                if (drawMode == DrawMode.Path)
                {
                    path.add(new Point(x, y)); // 將點加入到 path 中。
                }
                else if (drawMode == DrawMode.Rect)
                {
                    rect.setXY2(x, y);
                }
                else if (drawMode == DrawMode.Clip)
                {
                    clip.rect.setXY2(x, y);
                }
                board.redraw();
            }
        }

        public void boardMouseUp(int x, int y)
        {
            // 滑鼠放開時，繪製路徑 path 就結束了。
            if (drawMode == DrawMode.Path)
            {
                path.add(new Point(x, y));  // 將點加入到 path 中。
            }
            else if (drawMode == DrawMode.Rect)
            {
                rect.setXY2(x, y);
            }
            else if (drawMode == DrawMode.Clip)
            {
                clip.rect.setXY2(x, y);
                clip.image = clipScreen(clip.rect.r);
                Clipboard.SetImage(clip.image);
            }
            isMouseDown = false; // 滑鼠已經沒有被按下了
            board.redraw();
        }

        public void doAction(Action action)
        {
            setAction(action);

            switch (action)
            {
                case Action.Wait:
                    switchBoard(BoardType.Desktop);
                    break;
                case Action.Catch:
                    screenImage = screenCapture.CaptureScreen();
                    switchBoard(BoardType.Screen);
                    break;
                case Action.Clear:
                    records.Clear();
                    setPenColor(pen.Color); // 加入顏色記錄
                    setPenWidth((int) pen.Width); // 加入粗細記錄
                    board.redraw();
                    break;
                case Action.New:
                    break;
            }
        }

        public void switchBoard(BoardType pBoard)
        {
            boardType = pBoard;
            setBoardType(pBoard);

            if (pBoard == BoardType.Desktop)
            {
                board.Hide();
            }
            else
            {
                board.redraw();
                board.Show();
            }
        }
    }

    public class Path
    {
        List<Point> points = new List<Point>();

        public void add(Point p)
        {
            points.Add(p);
        }

        public void draw(Pen pen, Graphics g)
        {
            for (int i = 1; i < points.Count; i++)
            {
                g.DrawLine(pen, points[i-1], points[i]);
            }
        }
    }

    public class Rect
    {
        public Rectangle r = new Rectangle();

        public Rect(int x, int y, int w, int h)
        {
            r.X = x; r.Y = y; r.Width = w; r.Height = h;
        }

        public Rect setXY2(int x2, int y2) 
        {
            r.Width = x2-r.X; 
            r.Height = y2-r.Y;
            return this;
        }

        public void draw(Pen pen, Graphics g)
        {
            g.DrawRectangle(pen, r);
        }
    }

    public class Clip
    {
        public Bitmap image = null;
        public Rect rect = null;
    }
/*
    public class Catch
    {
        public Bitmap image = null;
    }
*/
    public class Width
    {
        public int width;
        public Width(int w) { width = w; }
    }
}
